use glib::subclass::prelude::ObjectSubclass;
use glib::subclass::prelude::*;
use glib::{ObjectExt, ParamSpec, ParamSpecInt64, ToValue};
use once_cell::sync::Lazy;
use transmission_client::StatsDetails;

use std::cell::Cell;

mod imp {
    use super::*;

    #[derive(Debug, Default)]
    pub struct TrSessionStatsDetails {
        pub seconds_active: Cell<i64>,
        pub downloaded_bytes: Cell<i64>,
        pub uploaded_bytes: Cell<i64>,
        pub files_added: Cell<i64>,
        pub session_count: Cell<i64>,
    }

    #[glib::object_subclass]
    impl ObjectSubclass for TrSessionStatsDetails {
        const NAME: &'static str = "TrSessionStatsDetails";
        type ParentType = glib::Object;
        type Type = super::TrSessionStatsDetails;
    }

    impl ObjectImpl for TrSessionStatsDetails {
        fn properties() -> &'static [ParamSpec] {
            static PROPERTIES: Lazy<Vec<ParamSpec>> = Lazy::new(|| {
                vec![
                    ParamSpecInt64::new(
                        "seconds-active",
                        "Seconds Active",
                        "Seconds Active",
                        0,
                        std::i64::MAX,
                        0,
                        glib::ParamFlags::READABLE,
                    ),
                    ParamSpecInt64::new(
                        "downloaded-bytes",
                        "Downloaded Bytes",
                        "Downloaded Bytes",
                        0,
                        std::i64::MAX,
                        0,
                        glib::ParamFlags::READABLE,
                    ),
                    ParamSpecInt64::new(
                        "uploaded-bytes",
                        "Uploaded Bytes",
                        "Uploaded Bytes",
                        0,
                        std::i64::MAX,
                        0,
                        glib::ParamFlags::READABLE,
                    ),
                    ParamSpecInt64::new(
                        "files-added",
                        "Files Added",
                        "Uploaded Bytes",
                        0,
                        std::i64::MAX,
                        0,
                        glib::ParamFlags::READABLE,
                    ),
                    ParamSpecInt64::new(
                        "session-count",
                        "Session Count",
                        "Session Count",
                        0,
                        std::i64::MAX,
                        0,
                        glib::ParamFlags::READABLE,
                    ),
                ]
            });
            PROPERTIES.as_ref()
        }

        fn property(&self, _obj: &Self::Type, _id: usize, pspec: &ParamSpec) -> glib::Value {
            match pspec.name() {
                "seconds-active" => self.seconds_active.get().to_value(),
                "downloaded-bytes" => self.downloaded_bytes.get().to_value(),
                "uploaded-bytes" => self.uploaded_bytes.get().to_value(),
                "files-added" => self.files_added.get().to_value(),
                "session-count" => self.session_count.get().to_value(),
                _ => unimplemented!(),
            }
        }
    }
}

glib::wrapper! {
    pub struct TrSessionStatsDetails(ObjectSubclass<imp::TrSessionStatsDetails>);
}

impl TrSessionStatsDetails {
    pub fn seconds_active(&self) -> i64 {
        self.property("seconds-active")
    }

    pub fn downloaded_bytes(&self) -> i64 {
        self.property("downloaded-bytes")
    }

    pub fn uploaded_bytes(&self) -> i64 {
        self.property("uploaded-bytes")
    }

    pub fn files_added(&self) -> i64 {
        self.property("files-added")
    }

    pub fn session_count(&self) -> i64 {
        self.property("session-count")
    }

    pub(crate) fn refresh_values(&self, rpc_details: StatsDetails) {
        let imp = imp::TrSessionStatsDetails::from_instance(self);

        // seconds_active
        if imp.seconds_active.get() != rpc_details.seconds_active {
            imp.seconds_active.set(rpc_details.seconds_active);
            self.notify("seconds-active");
        }

        // downloaded_bytes
        if imp.downloaded_bytes.get() != rpc_details.downloaded_bytes {
            imp.downloaded_bytes.set(rpc_details.downloaded_bytes);
            self.notify("downloaded-bytes");
        }

        // uploaded_bytes
        if imp.uploaded_bytes.get() != rpc_details.uploaded_bytes {
            imp.uploaded_bytes.set(rpc_details.uploaded_bytes);
            self.notify("uploaded-bytes");
        }

        // files_added
        if imp.files_added.get() != rpc_details.files_added {
            imp.files_added.set(rpc_details.files_added);
            self.notify("files-added");
        }

        // session_count
        if imp.session_count.get() != rpc_details.session_count {
            imp.session_count.set(rpc_details.session_count);
            self.notify("session-count");
        }
    }
}

impl Default for TrSessionStatsDetails {
    fn default() -> Self {
        glib::Object::new(&[]).unwrap()
    }
}
